import optparse
import os
import logging
import sys
from OpenSSL import crypto
from autoca import ca
from autoca import ca_stub
from autoca import certutil


def writeout(output, data):
    if output:
        outfd = open(output, 'w')
    else:
        outfd = sys.stdout
    outfd.write(data)
    if output:
        outfd.close()


def main():
    parser = optparse.OptionParser(usage='''%prog [<OPTIONS>] <COMMAND>

Known commands:

init
        Initialize a new local CA. You must provide the --ca-path
        and --ca-subject options.

get-ca
        Outputs the CA certificate in PEM format.

get-crl
        Outputs the CA Certificate Revocation List. It is possible
        to request a specific output format by passing the desired
        format as second argument ("pem" or "der").

update-crl
        Update the CRL on a local CA (requires --ca-path).

sign
        Create a new certificate request and sign it. If the --server
        option is not specified, a client certificate will be
        generated by default.

The tool can operate either with a local CA or a remote one. Use the
--ca-path option in the former case, or specify the server location
with --url in the latter. Accessing a remote CA might require
authentication: use the --secret option for puny shared-secret HTTP
auth (the default). Other authentication methods may be available on
your system.

X509 subjects, where applicable, must be specified as key=value pairs
separated by slashes ('/'), for example:

    CN=example.com/OU=Web/O=Example

''')
    g = parser.add_option_group('Connection options')
    g.add_option('--url', dest='url',
                 help='autoca API endpoint')
    g.add_option('--ca-path', dest='ca_path',
                 help='local CA directory')
    g = parser.add_option_group('Output options')
    g.add_option('--output', dest='output', metavar='FILE',
                 help='write output to this file')
    g.add_option('--outkey', dest='outkey', metavar='FILE',
                 help='write private key to this file (only '
                 'useful with the "sign" command)')
    g = parser.add_option_group('Options for the "init" command')
    g.add_option('--ca-subject', dest='ca_subject',
                      help='CA X.509 subject (only on initialization)')
    g.add_option('--ca-bits', dest='ca_bits', type='int', default=1024,
                      help='CA key size (only on initialization)')
    g = parser.add_option_group('Options for the "sign" command')
    g.add_option('--server', dest='server', action='store_true',
                      help='create a server certificate (for "sign")')
    g.add_option('--subject', dest='subject',
                      help='specify the X.509 subject as a set of '
                      'comma-separated ATTR=VALUE assignments')
    g.add_option('--secret', dest='secret',
                      help='shared secret for authentication')
    g.add_option('--days', dest='days', type='int', default=7,
                      help='certificate validity')
    opts, args = parser.parse_args()
    if len(args) < 1:
        parser.error('No command specified')
    if not opts.url and not opts.ca_path:
        parser.error('Must specify one of --url or --ca-path')
    if opts.url and opts.ca_path:
        parser.error('Must specify at most one of --url or --ca-path')

    secret = opts.secret
    if not secret:
        if os.getenv('AUTOCA_SECRET'):
            with open(os.getenv('AUTOCA_SECRET'), 'r') as fd:
                secret = fd.read().strip()

    if opts.url:
        client = ca_stub.CaStub(opts.url, secret)
    else:
        ca_subject = None
        if opts.ca_subject:
            ca_subject = certutil.parse_subject(opts.ca_subject)
        client = ca.CA(opts.ca_path, ca_subject, opts.ca_bits)

    cmd, args = args[0], args[1:]
    if cmd == 'get-ca':
        writeout(opts.output, client.get_ca())
    elif cmd == 'get-crl':
        fmt = 'pem'
        if args:
            fmt = args[0].lower()
        writeout(opts.output, client.get_crl(format=fmt))
    elif cmd == 'sign':
        if not opts.subject:
            parser.error('Must specify --subject')
        subject = certutil.parse_subject(opts.subject)
        pkey, cert = client.make_certificate(subject, days=opts.days,
                                             server=opts.server)
        writeout(opts.output, crypto.dump_certificate(
                crypto.FILETYPE_PEM, cert))
        writeout(opts.outkey, crypto.dump_privatekey(
                crypto.FILETYPE_PEM, pkey))
    elif cmd == 'init':
        if not opts.ca_subject:
            parser.error('Must specify --ca-subject')
        if not opts.ca_path:
            parser.error('The "init" command is only valid with a local CA')
        # Nothing to do, actually.
        print 'Done.'
    elif cmd == 'update-crl':
        if not opts.ca_path:
            parser.error('The "update-crl" command is only valid with a local CA')
        client.update_crl()
    else:
        parser.error('Unknown command')


if __name__ == '__main__':
    main()
